@@ -37,7 +37,7 @@ def sqlite_s3_query_multi(url, get_credentials=lambda now: (
     SQLITE_DONE = 101
     SQLITE_TRANSIENT = -1
     SQLITE_OPEN_READONLY = 0x00000001
-    SQLITE_OPEN_URI = 0x00000040
+    SQLITE_OPEN_NOMUTEX = 0x00008000
     SQLITE_IOCAP_IMMUTABLE = 0x00002000
 
     bind = {
@@ -141,7 +141,7 @@ def sqlite_s3_query_multi(url, get_credentials=lambda now: (
     def get_vfs(http_client):
         with make_auth_request(http_client, 'HEAD', (), ()) as response:
             head_headers = response.headers
-            next(response.iter_bytes())
+            next(response.iter_bytes(), b'')
 
         try:
             version_id = head_headers['x-amz-version-id']
@@ -189,7 +189,7 @@ def sqlite_s3_query_multi(url, get_credentials=lambda now: (
 
             return SQLITE_OK
 
-        x_file_size_type = CFUNCTYPE(c_int, c_void_p, POINTER(c_int))
+        x_file_size_type = CFUNCTYPE(c_int, c_void_p, POINTER(c_int64))
         def x_file_size(p_file, p_size):
             p_size[0] = size
             return SQLITE_OK
@@ -273,7 +273,7 @@ def sqlite_s3_query_multi(url, get_credentials=lambda now: (
     @contextmanager
     def get_db(vfs):
         db = c_void_p()
-        run(libsqlite3.sqlite3_open_v2, b'file:/' + file_name, byref(db), SQLITE_OPEN_READONLY | SQLITE_OPEN_URI, vfs_name)
+        run(libsqlite3.sqlite3_open_v2, file_name, byref(db), SQLITE_OPEN_READONLY | SQLITE_OPEN_NOMUTEX, vfs_name)
         try:
             yield db
         finally:
@@ -341,29 +341,31 @@ def sqlite_s3_query_multi(url, get_credentials=lambda now: (
                 for i in range(0, len(columns))
             )
 
-    def query(db, get_pp_stmts, sql, params=()):
-        for get_pp_stmt, finalize_stmt in get_pp_stmts(sql):
-            try:
-                pp_stmt = get_pp_stmt()
-                for i, param in enumerate(params):
-                    run_with_db(db, bind[type(param)], pp_stmt, i + 1, param)
+    def query(vfs, sql, params=()):
+        with \
+                get_db(vfs) as db, \
+                get_pp_stmt_getter(db) as get_pp_stmts:
 
-                columns = tuple(
-                    libsqlite3.sqlite3_column_name(pp_stmt, i).decode()
-                    for i in range(0, libsqlite3.sqlite3_column_count(pp_stmt))
-                )
+            for get_pp_stmt, finalize_stmt in get_pp_stmts(sql):
+                try:
+                    pp_stmt = get_pp_stmt()
+                    for i, param in enumerate(params):
+                        run_with_db(db, bind[type(param)], pp_stmt, i + 1, param)
 
-                yield columns, rows(get_pp_stmt, columns)
-            finally:
-                finalize_stmt()
+                    columns = tuple(
+                        libsqlite3.sqlite3_column_name(pp_stmt, i).decode()
+                        for i in range(0, libsqlite3.sqlite3_column_count(pp_stmt))
+                    )
+
+                    yield columns, rows(get_pp_stmt, columns)
+                finally:
+                    finalize_stmt()
 
     with \
             get_http_client() as http_client, \
-            get_vfs(http_client) as vfs, \
-            get_db(vfs) as db, \
-            get_pp_stmt_getter(db) as get_pp_stmts:
+            get_vfs(http_client) as vfs:
 
-        yield partial(query, db, get_pp_stmts)
+        yield partial(query, vfs)
 
 
 @contextmanager
